/*
****************************************************************************
File Name: main.c

This program and associated design is for testing communnication between 
Cypress Semicinductor PSoC 5LP KIT-059 and AD75019 Crosspoint Switch 

******************************************************************************
 Copyright (2022), LMA  Pandrol / Vortok Condition Monitoring
******************************************************************************
*/

/* 
Functional Overview

This circuit uses a Cypress Semicinductor PSoC 5LP KIT-059 to 
communicate switch open and closures to an AD75019 Crosspoint Switch

Eight pushbuttons are used to adjust D to A digial settings to control outputs. 
switches 1 through 7 are soft switches - what they do depends on the operational mode
pushing any of them puts the circuit into an adjustment mode
switch 8 is the exit switch - it ends any adjustment mode and returns to display mode

display mode shows the parameters of the output signals
adjustment mode activates the soft switches to change the various parameters

switch   type   function 
  1       NO    soft key  dataum
  2       NO    soft key  engine
  3       NO    soft key  car 
  4       NO    soft key  coach count
  5       NO    soft key  headway
  6       NO    soft key  speed
  7       NO    soft key  passenger/freight   
  8       NO    exit menu and operate 

*/

#include "project.h"
#include "stdio.h"
#include "string.h" 
#include "math.h"


/* comment out things not used
// defines the debounce time in milliseconds, max value is 255 
#define  DEBOUNCE_TIME  50

// ISR function prototype declarations 
CY_ISR_PROTO(Tick_ISR);
CY_ISR_PROTO(PICU_ISR);

// register bit masks for the switches 
// comment out switch masks
#define  SW1_MASK  1
#define  SW2_MASK  2
#define  SW3_MASK  4
#define  SW4_MASK  8
#define  SW5_MASK  16
#define  SW6_MASK  32
#define  SW7_MASK  64
#define  SW8_MASK  128

#define  TIMED_OUT  0

// switch debounce timeout variables decremented by Tick_ISR(),
// and monitored and reset by PICU_ISR()
static volatile uint8 CYDATA switch_1_timeout;
static volatile uint8 CYDATA switch_2_timeout;
static volatile uint8 CYDATA switch_3_timeout;
static volatile uint8 CYDATA switch_4_timeout;
static volatile uint8 CYDATA switch_5_timeout;
static volatile uint8 CYDATA switch_6_timeout;
static volatile uint8 CYDATA switch_7_timeout;
static volatile uint8 CYDATA switch_8_timeout;
end comment out things not used  */

uint16   i, ii, j, jj, k, kk;             //index variables
uint16   pressed_switch ;                 //which switch was pressed? 

uint8   V0          = 100;   //left 
uint8   V1          = 75;    //datum - no train voltage to WheelChex 
uint8   V2          = 150;   //right
uint8   carcount    = 6;     //number of cars in train
//for test purposes headway 2 minutes or 8 units of 15 seconds
uint8   headway     = 8;     //15 second headway - 1 units of 15 second intervals                                                                                                                                                                                                                                                                                                                                                                                                                                                                                f 15 sec each
uint8   endheadway  = 0;     //flag to end headway at property change
                             //1 = end, 0 = continue

uint8   endmode     = 0;    //0 = endmode 1 = continue mode

//times for ensuring wheel pattern looks like train 
//based on M7 commuter train 
//0.1 ms per count - use CyDelayUS(count * 100) to get 0.1 ms or 100 us
uint8   speedindex = 1;      //index variable set to 25 mph
uint8   maxspeedindex = 8;   //0 to 8, for 9 values
uint8   speed[8]   = {10, 25, 40, 50, 60, 75, 100, 150};    //train speeds: 10, 25, 40, 50, 60, 75, 100, 150 mph 
                                                            //array always starts wth subscript 0

//8 bit 0 to 255     16 bits 0 to 65535
//   speed               10     25   40   50   60    75   100  150 
uint16  baytime[8] =  { 102,   41,  26,  20,  17,   14,   10,   7};    //time wheel is in bay 
uint16  w2wtime[8] =  { 580,  232, 145, 116,  97,   77,   58,  39};    //time between wheels in bay
uint16  t2ttime[8] =  {3477, 1391, 869, 695, 580,  464,  348, 232};    //time from second wheel of truck to first wheel of next truck same car
uint16  car2car[8] =  {1159,  464, 290, 232, 193,  155,  116,  77};    //time from last wheel of last truck of car 1 to first wheel of first truck of car 2
uint16  tietime[8] =  {  34,   14,   9,   7,   6,    5,    3,   2};    //time wheel is in tie 

uint8   passfreight  = 0;       // 0 = passenger with no engine
                                // 1 = freight with engine

//slope and intercept give conversion from 0 to 255 range of voltages put out by 
//DACs to voltages put out to WheelChex
//set slope and intercept for test purposes
float   slope     = 4.0/255;     //slope and intercept for converting
float   intercept = 0 ;          //counts to output voltages 0 for WC1, -10 for WC2

double  voltage   =  0.0 ;
char    sign;                   //sign of voltage                                         
double  whole     = 0;          //integer part of voltage
double  frac      = 0;          //fractional part

/*
crosspoint switch controls words to transmit
256 bits per transmission
in eight 32 bit words 
or 16    16 bit words
*/

// DMA Configuration for DMA_TX 
//16 2 byte words - 2 bytes/burst 
//2 by 16 is 32 - TD transfer count is 32 - 16 two byte words
#define DMA_TX_BYTES_PER_BURST      (32)    //bytes per burst - here, 256 bit / 8 bits per byte = 32 bytes
#define DMA_TX_REQUEST_PER_BURST    (0)     //0 for automatic bursts 1 for request per burst
#define DMA_TX_SRC_BASE             (CYDEV_SRAM_BASE)
#define DMA_TX_DST_BASE             (CYDEV_PERIPH_BASE)

#define BUFFER_SIZE                 (16u)
#define STORE_TD_CFG_ONCMPLT        (1u)

/* Variable declarations for DMA_TX 
   one DMA channel for each buffer data set */

uint8 txChannel_0, txChannel_1,txChannel_2,txChannel_3,txChannel_4,txChannel_51,txChannel_62 ;
uint8 txChannel_73,txChannel_84,txChannel_5,txChannel_6,txChannel_7,txChannel_8;
uint8 txTD_0, txTD_1,  txTD_2,  txTD_3, txTD_4, txTD_51, txTD_62  ; 
uint8 txTD_5, txTD_73, txTD_84, txTD_6, txTD_7, txTD_8 ;

/* Data buffers */
//numbers after tx are the bay that data applies to
//for example tx_0 is no wheel. applies to all bays
//tx_2 applies to wheel in bay 2   tx_73 applies to 1st wheel in bay 7, 2nd wheel in bay 3
//0x indicates that the next digits are hex 0 - F
//0x0001 is 0000 0000 0000 0001 in binary - 16 bits
//16 bits x 16 positions is 256
//                                          F        E         D        C        B        A        9        8        7        6        5        4        3        2        1        0
const uint8 tx_0_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u};
const uint8 tx_1_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u};
const uint8 tx_2_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u};
const uint8 tx_3_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u};
const uint8 tx_4_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u};
const uint8 tx_51_Buffer [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0002u};
const uint8 tx_62_Buffer [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0000u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u};
const uint8 tx_73_Buffer [BUFFER_SIZE] = {0x0001u, 0x0004u,  0x0001u, 0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u};
const uint8 tx_84_Buffer [BUFFER_SIZE] = {0x0004u, 0x0001u,  0x0001u, 0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u};
const uint8 tx_5_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0004u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0001u};
const uint8 tx_7_Buffer  [BUFFER_SIZE] = {0x0001u, 0x0004u,  0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u};
const uint8 tx_8_Buffer  [BUFFER_SIZE] = {0x0004u, 0x0001u,  0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0002u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u};

//generic buffer:
//uint8 tx_Buffer [BUFFER_SIZE] = {0x0001u, 0x0001u,  0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x00001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u, 0x0001u};
//initialize
//tx_Buffer [BUFFER_SIZE] = tx_0_Buffer [BUFFER_SIZE];



/*****************************************************************************
* Function Name: Tick_ISR
******************************************************************************
* Summary: Interrupt once per millisecond. 
*
* Parameters: none; it's an ISR function
*
* Return: none; it's an ISR function
*
*****************************************************************************/

/*comment out CY_ISR(Tick_ISR) switch subroutines 
CY_ISR(Tick_ISR)
{
	if(switch_1_timeout != 0)
	{ 
        //Come here if an edge was detected by the PICU.
		//Decrement and check timeout.
		
		if(--switch_1_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW1_MASK) == 0) pressed_switch = 1;
		}
	}

	// repeat for switch 2 
	if(switch_2_timeout != 0)
	{
		if(--switch_2_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW2_MASK) == 0) pressed_switch = 2;
		}
	}
    
    // repeat for switch 3 
	if(switch_3_timeout != 0)
	{
		if(--switch_3_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW3_MASK) == 0) pressed_switch = 3;
		}
	}
    
    // repeat for switch 4 
	if(switch_4_timeout != 0)
	{
		if(--switch_4_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW4_MASK) == 0) pressed_switch = 4; 
		}
	}
    
    //  repeat for switch 5 
	if(switch_5_timeout != 0)
	{
		if(--switch_5_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW5_MASK) == 0) pressed_switch = 5;
		}
	}
    
    //  repeat for switch 6 
	if(switch_6_timeout != 0)
	{
		if(--switch_6_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW6_MASK) == 0) pressed_switch = 6; 
		}
	}
    
    //  repeat for switch 7 
	if(switch_7_timeout != 0)
	{
		if(--switch_7_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW7_MASK) == 0) pressed_switch = 7;
		}
	}
    
    //  repeat for switch 8 
	if(switch_8_timeout != 0)
	{
		if(--switch_8_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW8_MASK) == 0) pressed_switch = 8; 
		}
	} //end if  

}  //end tick isr
end comment out CY_ISR switch subroutines */

/*****************************************************************************
* Function Name: PICU_ISR
******************************************************************************
* Summary: Interrupt only occurs on falling edge of either pin, i.e., when
*		   switch is pressed. Checks and initiates debounce timeout.
*
* Parameters: none; it's an ISR function
*
* Return: none; it's an ISR function
*
*****************************************************************************/
/*comment out CY_ISR(PICU_ISR) switch subroutines 
CY_ISR(PICU_ISR)
{
	//  copies of PICU registers 
	uint8 CYDATA temp_stat;
	
	//  read the PICU interrupt status register, with a clear on read 
	temp_stat = Switches_ClearInterrupt();
    endheadway = 1;

	//  Process the PICU event on SW1 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW1_MASK) != 0) && (switch_1_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_1_timeout = DEBOUNCE_TIME; 
	}
	
	//  Process the PICU event on SW2 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW2_MASK) != 0) && (switch_2_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_2_timeout = DEBOUNCE_TIME; 
	}
    
    //  Process the PICU event on SW3 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW3_MASK) != 0) && (switch_3_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_3_timeout = DEBOUNCE_TIME; 
	}
	
	//  Process the PICU event on SW4 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW4_MASK) != 0) && (switch_4_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_4_timeout = DEBOUNCE_TIME; 
	}
    
    //  Process the PICU event on SW5 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW5_MASK) != 0) && (switch_5_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_5_timeout = DEBOUNCE_TIME; 
	}
	
	//  Process the PICU event on SW6 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW6_MASK) != 0) && (switch_6_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_6_timeout = DEBOUNCE_TIME; 
	}
    
    //  Process the PICU event on SW7 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW7_MASK) != 0) && (switch_7_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_7_timeout = DEBOUNCE_TIME; 
	}
	
	//  Process the PICU event on SW8 only if any
	//  ongoing debounce period has timed out 
	if(((temp_stat & SW8_MASK) != 0) && (switch_8_timeout == TIMED_OUT))
	{
		//  reset the debounce timer for this button 
		switch_8_timeout = DEBOUNCE_TIME; 
	}
}  //end picu isr
end comment out CY_ISR(PICU_ISR) switch subroutines */


// function: 
void lma_clear_display(void) 
{
    //clear dispaly 
    for(j = 0; j<=3; j++)
    {
        LCD_Position(j,0);
        LCD_PrintString("                    ");
    }
}  //end clear display for(j = 0; j<=3; j++) 

// function: 
void trainstart() 
{
    //scope pulse at atart of train
    TrainStartPulse_Write(1);
    CyDelay(2);
    TrainStartPulse_Write(0);
}

// function: 
void trainstop() 
{
    //scope pulse at end of train
    TrainStopPulse_Write(1);
    CyDelay(2);
    TrainStopPulse_Write(0);
}

// function: 
void xp_parallel_clock() 
{
    //pulse to parallel load xp switch
    xp_parallel_clk_Write(1);
    CyDelay(2);
    xp_parallel_clk_Write(0);
}

/*comment out change_volt subroutines 
//function: 
uint8 change_volt(uint8 lma_type, uint8 lma_value) 

//change voltage for datum, engine and car
//pass in type - left, datum, or right
//pass in present value thereof 
//return new value thereof
{
    while(endmode)
    {
        //show present datum in volts
        lma_clear_display();

        LCD_Position(0,0);
        if (lma_type == 1) LCD_PrintString("left  is ");   //V0
        if (lma_type == 2) LCD_PrintString("datum is ");   //V1
        if (lma_type == 3) LCD_PrintString("right is ");   //V2
        
        LCD_PrintNumber(lma_value);
        LCD_PrintString(" ");
        
        voltage = (lma_value * slope) + intercept;
        sign = '+';
        if (voltage < 0.0)
        {
            sign = '-' ;
            voltage = voltage * -1;
        }
        frac = (100*modf(voltage, &whole));            
        LCD_PrintString(&sign); 
        LCD_PrintU32Number(whole);
        LCD_PrintString(".");
        if (frac < 10) LCD_PrintString("0");
        LCD_PrintU32Number(frac);

        voltage = (25 * slope);
        frac = (100*modf(voltage, &whole));

        LCD_Position(1,0); 
        LCD_PrintString("-     25 " ); 
        //LCD_Position(1,8); 

        LCD_PrintU32Number(whole);
        LCD_PrintString(".");
        if (frac < 10) LCD_PrintString("0");
        LCD_PrintU32Number(frac);

        LCD_Position(1,18);
        LCD_PrintString("+" ); 
    
        //+/- 10 counts
        voltage = (10 * slope) ;
        frac = (100*modf(voltage, &whole));
              
        LCD_Position(2,0); 
        LCD_PrintString("-     10 " );
        //LCD_Position(2,8); 

        LCD_PrintU32Number(whole);
        LCD_PrintString(".");
        if (frac < 10) LCD_PrintString("0");
        LCD_PrintU32Number(frac);

        LCD_Position(2,18);
        LCD_PrintString("+" ); 
    
        //+/- 1 counts
        voltage = (1 * slope) ;
        frac = (100*modf(voltage, &whole));
              
        LCD_Position(3,0); 
        LCD_PrintString("-      1 " ); 
       // LCD_Position(3,8); 
        LCD_PrintU32Number(whole);
        LCD_PrintString(".");
        if (frac < 10) LCD_PrintString("0");
        LCD_PrintU32Number(frac);
    
        LCD_Position(3,18);
        LCD_PrintString("+" ); 
    
        if (pressed_switch != 0)
        {
            pressed_switch = pressed_switch ;
            endmode = 1; //set to loop though changes
            switch(pressed_switch)
            {
                //act on pressed switch 
                case 1:
                    //  
                    pressed_switch = 0 ;
                    lma_value = lma_value - 25;
                    //if (weight0 <= 253) weight0 = weight0 + 2; 
                    break;
                case 2:
                    //change engine level
                    pressed_switch = 0 ;
                    lma_value = lma_value - 10;
                    //if (weight0 >= 2) weight0 = weight0 - 2;  
                    break;
                case 3:
                    //change car level 
                    pressed_switch = 0 ;
                    lma_value = lma_value - 1;
                    //if (weight1 <= 253) weight1 = weight1 + 2;
                    break;
                case 4:
                    //not used
                    pressed_switch = 0 ;
                    //if (weight1 >= 2) weight1 = weight1 - 2;
                    break;
                case 5:
                    //change headway 
                    pressed_switch = 0 ;
                    lma_value = lma_value + 25;
                    //if (weight2 <= 253) weight2 = weight2 + 2;
                    break;
                case 6:
                    //change speed 
                    pressed_switch = 0 ;
                    lma_value = lma_value + 10;
                    //if (weight2 >= 2) weight2 = weight2 - 2;
                    break;  
                case 7:
                    //passenger or freight
                    //freight has no engine
                    pressed_switch = 0 ;
                    lma_value = lma_value + 1;
                    break;
                case 8:
                    //done
                    pressed_switch = 0 ;
                    endmode = 0;  //end loop through changes
                    break;     
            }//end case
        }//end pressed switch
    }//end while
    return lma_value;
}//end change voltage


//function:
void setdatum() //change datum value
{
    while(endmode)
    {
        V1 = change_volt(1, V1);    
    }//end while
}//end set datum

//function:
void setleft() //change engine value
{
    while(endmode)
    {
        V0 = change_volt(2, V0);    
    }//end while
}//end set datum

//function:
void setright() //change car value
{
    while(endmode)
    {
        V2 = change_volt(3, V2);    
    }//end while
}//end set car
end comment out change_volt subroutines */

/*comment out various setting subroutines 
//function:
void setcarcount() //change number of cars
{
    while(endmode)
    {
        lma_clear_display();
        //show car count 
        LCD_Position(0,0);
        LCD_PrintString("car count is ");   
        LCD_PrintNumber(carcount);

        //show increase/decrease steps
        //+/- 10 counts
        LCD_Position(1,0); 
        LCD_PrintString("-10              +10" ); 

        //+/- 5 counts
        LCD_Position(2,0); 
        LCD_PrintString("-5                +5" ); 

        //+/- 1 counts
        LCD_Position(3,0); 
        LCD_PrintString("-1                +1" ); 
   
        if (pressed_switch != 0)
        {
            switch(pressed_switch)
            {
                //act on pressed switch 
                case 1:
                    pressed_switch = 0 ;
                    if (carcount > 10) carcount = carcount- 10;
                    break;
                case 2:
                    //change engine level
                    pressed_switch = 0 ;
                    if (carcount > 5) carcount = carcount - 5;
                    break;
                case 3:
                    //change car level 
                    pressed_switch = 0 ;
                    if (carcount > 1) carcount = carcount - 1;
                    break;                    break;
                case 4:
                    //not used
                    pressed_switch = 0 ;
                    break;
                case 5:
                    //change headway 
                    pressed_switch = 0 ;
                    if (carcount + 10 < 98) carcount = carcount + 10;
                    break;                 
                case 6:
                    //change speed 
                    pressed_switch = 0 ;
                    if (carcount + 5 < 98) carcount = carcount + 5;
                    break;                                    
                case 7:
                    //passenger or freight
                    pressed_switch = 0 ;
                    if (carcount + 1 < 98) carcount = carcount + 1;
                    break;                        
                case 8:
                    //done
                    pressed_switch = 0 ;
                    endmode = 0;  //end loop through changes
                    break;     
            }//end switch
        }//end if pressed switch
    }//end while
}//end set car count

//function:
void setheadway() //change headway - time between trains
{
    while(endmode)
    {
        lma_clear_display();
    
        //show present headway 
        //headway is in 15 second intervals
        //headway = 4 is 1 minute
        //headway = 40 is 10 min     = 120 is 30 min
        LCD_Position(0,0);
        LCD_PrintString("headway is ");
    
        LCD_Position(0,11);
        voltage = (headway *15.0)/60.0;    //headway in minutes 
        frac = (100*modf(voltage, &whole));
        LCD_PrintU32Number(whole);
        LCD_PrintString(".");
        if (frac < 10) LCD_PrintString("0");
        LCD_PrintU32Number(frac);
    
        LCD_Position(0,17);
        LCD_PrintString("min");

        //show increase/decrease steps
        //+/- 10 counts
        LCD_Position(1,0); 
        LCD_PrintString("-5     minutes    +5" ); 

        //+/- 5 counts
        LCD_Position(2,0); 
        LCD_PrintString("-1     minutes    +1" ); 

        //+/- 1 counts
        LCD_Position(3,0); 
        LCD_PrintString("-1   15 seconds   +1" ); 
   
        if (pressed_switch != 0)
        {
            switch(pressed_switch)
            {
                //act on pressed switch 
                case 1:
                    pressed_switch = 0 ;
                    if (headway > 20) headway = headway - 20;
                    break;
                case 2:
                    //change engine level
                    pressed_switch = 0 ;
                    if (headway > 4) headway = headway - 4;
                    break;
                case 3:
                    //change car level 
                    pressed_switch = 0 ;
                    if (headway > 1) headway = headway - 1;
                    break;           
                case 4:
                    //not used
                    pressed_switch = 0 ;
                    break;
                case 5:
                    //change headway 
                    pressed_switch = 0 ;
                    if (headway < 248) headway = headway + 20;
                    //248 15 sec intervals is 3720 sec or 62 minutes
                    break;                 
                case 6:
                    //change speed 
                    pressed_switch = 0 ;
                    if (headway < 248) headway= headway + 4;
                    break;                                    
                case 7:
                    //passenger or freight
                    pressed_switch = 0 ;
                    if (headway < 248) headway= headway + 1;
                    break;                        
                case 8:
                    //done
                    pressed_switch = 0 ;
                    endmode = 0;  //end loop through changes
                    break;     
            }//end switch
        }//end if pressed switch
    }//end while
}//end set headway
    
//function:
void setspeed() //change speed of train
{
    while(endmode)
    {
        lma_clear_display();
        //show present speed 
        LCD_Position(0,0);
        LCD_PrintString("speed is ");
    
        LCD_Position(0,12);
        LCD_PrintNumber(speed[speedindex]);

        //show increase/decrease steps
        //+/- 1 speed step 
        LCD_Position(1,0); 
        LCD_PrintString("increase speed     " ); 

        //+/- 1 speed step 
        LCD_Position(2,0); 
        LCD_PrintString("decrease speed      " ); 
        
        if (pressed_switch != 0)
        {
            switch(pressed_switch)
            {
                //act on pressed switch 
                case 1:
                    pressed_switch = 0 ;
                    if (speedindex < maxspeedindex) speedindex ++;
                    break;
                case 2:
                    pressed_switch = 0 ;
                    if (speedindex > 0 ) speedindex --;
                    break;
                case 3:
                    //not used
                    pressed_switch = 0 ;
                    break;           
                case 4:
                    //not used
                    pressed_switch = 0 ;
                    break;           
                case 5:
                    //not used
                    pressed_switch = 0 ;
                    break;           
                case 6:
                    //not used
                    pressed_switch = 0 ;
                    break;           
                case 7:
                    //not used
                    pressed_switch = 0 ;
                    break;           
                case 8:
                    //done
                    pressed_switch = 0 ;
                    endmode = 0;  //end loop through changes
                    break;     
            }//end switch
        }//end if pressed switch
    }//end while
}//end set speed
end comment out change_volt subroutines */

//function:
void showstatus()  //display while mode = 0
{
    lma_clear_display();
    LCD_Position(0,0);
    LCD_PrintString(" lft   dat   rht  n ");   

    //dataum
    LCD_Position(1,0);
    voltage = (V0 * slope) + intercept;
    sign = '+';
    if (voltage < 0.0)
    {
        sign = '-' ;
        voltage = voltage * -1;
    }
    frac = (100*modf(voltage, &whole));
                  
    LCD_PrintString(&sign); 
    LCD_PrintU32Number(whole);
    LCD_PrintString(".");
    if (frac < 10) LCD_PrintString("0");
    LCD_PrintU32Number(frac);

    //engine
    LCD_Position(1,6);
    voltage = (V1 * slope) + intercept;
    sign = '+';
    if (voltage < 0.0)
    {
        sign = '-' ;
        voltage = voltage * -1;
    }
    frac = (100*modf(voltage, &whole));
    LCD_PrintString(&sign); 
    LCD_PrintU32Number(whole);
    LCD_PrintString(".");
    if (frac < 10) LCD_PrintString("0");
    LCD_PrintU32Number(frac);
                
    //car
    LCD_Position(1,12);
    voltage = (V2 * slope) + intercept;
    sign = '+';
    if (voltage < 0.0)
    {
        sign = '-' ;
        voltage = voltage * -1;
    }
    frac = (100*modf(voltage, &whole));
        
    LCD_PrintString(&sign); 
    LCD_PrintU32Number(whole);
    LCD_PrintString(".");
    if (frac < 10) LCD_PrintString("0");
    LCD_PrintU32Number(frac);
    
    //number of cars
    LCD_Position(1,18);
    LCD_PrintNumber(carcount);
    
    //headway: 
    LCD_Position(2,0);
    LCD_PrintString("headway ");
    
    LCD_Position(2,9);
    voltage = (headway *15.0)/60.0;    //headway in minutes 
    frac = (100*modf(voltage, &whole));
    LCD_PrintU32Number(whole);
    LCD_PrintString(".");
    if (frac < 10) LCD_PrintString("0");
    LCD_PrintU32Number(frac);
    
    LCD_Position(2,14);
    LCD_PrintString("min    ");

    //speed: 
    LCD_Position(3,0);
    LCD_PrintString("speed               ");
    
    LCD_Position(3,6);
    LCD_PrintU32Number(speed[speedindex]);
    LCD_PrintString(" ");
    LCD_Position(3,10);
    LCD_PrintString("mph ");   
    
    //passenger or freight
    LCD_Position(3,15);
    //if (passfreight == 0) LCD_PrintString("pass  ");   
    //if (passfreight == 1) LCD_PrintString("frght ");   
    LCD_PrintString("pass  ");
    
}

/* ******************************************************************************
* Function Name: DmaTxConfiguration
********************************************************************************
* Summary:
*  Configures the DMA transfer for TX direction
*   
* Parameters:
*  None.
*
* Return:
*  None.
*
* copied and edited from SPIM Example
*
*******************************************************************************/
void DmaTxConfiguration()
{
    /* Init DMA, 1 byte bursts, each burst requires a request */ 
    
    /* AN52705 section 5 firmware configuration of DMA
    item 1  start DMA channel
    DMA channel_handle is txChannel
    DMA_TX_DmaInitialize      is function call to do initialization
    DMA_TX_BYTES_PER_BURST    is number of bytes per burst - here, 256 bit / 8 bits per byte = 32 bytes
    DMA_TX_REQUEST_PER_BURST  is nubmre of DMA requests per burst - here, definced as 1u 
    HI16(DMA_TX_SRC_BASE)     is high 16 bits of source (SRC) address
    HI16(DMA_TX_DST_BASE))    is high 16 bits of destination address (DST) address
    */
    txChannel_0 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_1 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_2 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_3 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_4 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_51=  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_62 = DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_73 = DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_84 = DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_5 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_6 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_7 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));
    txChannel_8 =  DMA_TX_DmaInitialize(DMA_TX_BYTES_PER_BURST, DMA_TX_REQUEST_PER_BURST, HI16(DMA_TX_SRC_BASE), HI16(DMA_TX_DST_BASE));

    /* AN52705 section 5 firmware configuration of DMA
    item 2  create an instance of a TD (Transaction Descriptor)
    TD_Handle is CyDmaTdAllocate()
    so TD_handel is txTD that is, transmit transaction descriptor
    */
    txTD_0  = CyDmaTdAllocate();
    txTD_1  = CyDmaTdAllocate();
    txTD_2  = CyDmaTdAllocate();
    txTD_3  = CyDmaTdAllocate();
    txTD_4  = CyDmaTdAllocate();
    txTD_51 = CyDmaTdAllocate();
    txTD_62 = CyDmaTdAllocate();
    txTD_73 = CyDmaTdAllocate();
    txTD_84 = CyDmaTdAllocate();
    txTD_5  = CyDmaTdAllocate();
    txTD_6  = CyDmaTdAllocate();
    txTD_7  = CyDmaTdAllocate();
    txTD_8  = CyDmaTdAllocate();
    
    /* Configure this Td as follows:
    *  - Increment the source address, but not the destination address   
    */
    /* AN52705 section 5 firmware configuration of DMA
    item 3  set TD configuration
    CyDmaTdSetConfiguration(TD_Handle, Transfer_Count, Next_TD, TD_Property)
    CyDmaTdSetConfiguration is function call
    txTD                    is TD handle
    BUFFER_SIZE             is transfer count - the size of buffer, here, 16u
    Next_TD                 is CY_DMA_DISABLE_TD - disable dma channel when transfer complete
    TD_Property             is TD_INC_SRC_ADR - increment source address low 16 bits
    */
    CyDmaTdSetConfiguration(txTD_0,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_1,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_2,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_3,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_4,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_51, BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_62, BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_73, BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_84, BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_5,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_6,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_7,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    CyDmaTdSetConfiguration(txTD_8,  BUFFER_SIZE, CY_DMA_DISABLE_TD, TD_INC_SRC_ADR);
    
    /* From the memory to the SPIM */
    /* AN52705 section 5 firmware configuration of DMA
    item 4  set TD address
    CyDmaTdSetAddress(TD_Handle, LO16(Source Address), LO16(Destination Address))
    CyDmaTdSetAddress                is function call 
    txTD                             is transaction descriptor handle
    LO16((uint32)txBuffer)           is low 16 bits of source address 
    LO16((uint32) SPIM_TXDATA_PTR))  is low 16 bits of destination address
    */
    CyDmaTdSetAddress(txTD_0,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_1,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_2,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_3,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_4,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_51, LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_62, LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_73, LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_84, LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_5,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_6,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_7,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));
    CyDmaTdSetAddress(txTD_8,  LO16((uint32)tx_0_Buffer), LO16((uint32)SPIM_TXDATA_PTR));


    /* Associate the TD with the channel */
    /* AN52705 section 5 firmware configuration of DMA
    item 5  set channel's initial TD 
    CyDmaChSetInitialTd(Channel_Handle, TD_Handle)
    CyDmaChSetInitialTd is the function call
    Channel_Handle      is the DMA Channel Handle - txChannel (set in item 1 above) 
    TD_Handle           is the TD Handle - txTD (set in 2 above)
    */
    CyDmaChSetInitialTd(txChannel_0,  txTD_0); 
    CyDmaChSetInitialTd(txChannel_1,  txTD_1); 
    CyDmaChSetInitialTd(txChannel_2,  txTD_2); 
    CyDmaChSetInitialTd(txChannel_3,  txTD_3); 
    CyDmaChSetInitialTd(txChannel_4,  txTD_4); 
    CyDmaChSetInitialTd(txChannel_51, txTD_51); 
    CyDmaChSetInitialTd(txChannel_62, txTD_62); 
    CyDmaChSetInitialTd(txChannel_73, txTD_73); 
    CyDmaChSetInitialTd(txChannel_84, txTD_84); 
    CyDmaChSetInitialTd(txChannel_5,  txTD_5); 
    CyDmaChSetInitialTd(txChannel_6,  txTD_6); 
    CyDmaChSetInitialTd(txChannel_7,  txTD_7); 
    CyDmaChSetInitialTd(txChannel_8,  txTD_8); 
      
    
    //enable DMA channels
    CyDmaChEnable(txChannel_0,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_1,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_2,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_3,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_4,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_51, STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_62, STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_73, STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_84, STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_5,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_6,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_7,  STORE_TD_CFG_ONCMPLT);
    CyDmaChEnable(txChannel_8,  STORE_TD_CFG_ONCMPLT);

}    
int main(void)
{
    // Initialize the two custom defined ISRs 
    //isr_1_StartEx(PICU_ISR);
    //isr_2_StartEx(Tick_ISR);
    
    
    //initialize display
    LCD_Start();
    TrainStartPulse_Write(0);
    TrainStopPulse_Write(0);

    CyGlobalIntEnable; // Enable global interrupts. 

    //start up VDACs
    //VDAC 0 and 1 for WWheelChex Amplifier Rack Simulation

    VDAC_0_Start();
    VDAC_1_Start();
    VDAC_2_Start();
    
    VDAC_0_SetSpeed(VDAC_0_HIGHSPEED);
    VDAC_1_SetSpeed(VDAC_1_HIGHSPEED);
    VDAC_2_SetSpeed(VDAC_2_HIGHSPEED);
        
    VDAC_0_SetRange(VDAC_0_RANGE_4V);
    VDAC_1_SetRange(VDAC_1_RANGE_4V);
    VDAC_2_SetRange(VDAC_2_RANGE_4V);

    //enable DMA transmit channel
    CyDmaChEnable(txChannel_0, STORE_TD_CFG_ONCMPLT);

    
    //set starting value = 0V   
    VDAC_0_SetValue(V0);          //fixed values for testing
    VDAC_1_SetValue(V1);
    VDAC_2_SetValue(V2);      
    
    
    /* comment out intercept
    //set intercept value: 
    if (WC12_Read() == 0) intercept = 0.0;   //datum - no train voltage, 7 V, to WheelChex 1
    else intercept = 128;                    //datum - no train voltage, 0 V to WheelChex 2
    
    
    if (WC12_Read() == 0) V1 = 75;    //datum - no train voltage, 7 V, to WheelChex 1
    else V1 = 100;                    //datum - no train voltage, 0 V to WheelChex 2
    
    end comment out intercept  */

    //initialize DMA
    DmaTxConfiguration();
    
    //start SPIM
    SPIM_Start();
    SPIM_EnableTxInt();
    SPIM_TxEnable();    
    
    
    
    while(1)      //continuous loop through program 
    {      
        //if (mode == 0) showstatus();  //show status
        showstatus();
       
        /* comment out changes 
        if (pressed_switch != 0)
        {
            //pressed_switch = pressed_switch ;
            endmode = 1; //set to loop though changes
            
            switch(pressed_switch)
            {
                //act on pressed switch 
                case 1:
                    //change left level  
                    pressed_switch = 0 ;
                    setleft();
                    break;
                 case 2:
                    //change datum level
                    pressed_switch = 0 ;
                    setdatum();
                    break;
                case 3:
                    //change right level 
                    pressed_switch = 0 ;
                    setright();
                    break;
                case 4:
                    //change car count
                    pressed_switch = 0 ;
                    setcarcount();
                    break;
                case 5:
                    //change headway 
                    pressed_switch = 0 ;
                    setheadway();
                    break;
                case 6:
                    //change speed 
                    pressed_switch = 0 ;
                    setspeed();
                    break;  
                case 7:
                    //passenger or freight
                    //passenger has no engine
                    //passfreight = 1 is freight train
                    //            = 0 is passenger
                    pressed_switch = 0 ;
                    passfreight = 1 - passfreight;
                    endmode = 0;
                    break;
                case 8:
                    //done
                    pressed_switch = 0 ;
                    endmode = 0;  //end loop through changes
                    break;     
                default:
                    LCD_PrintString("DEFAULT case       ");
                    break;   
            }  //end switch
        } //end if pressed switch
        end comment out changes */

     
        
        
        
        //train pattern: 
        //4 engine wheels
        
        
        /* comment out passenger or freight 
            if(passfreight) 
            {
                for(j = 0; j<=3; j++)
                {
                    // scope trigger
                    if (j == 0) trainstart();
                 
                    // engine 
                    //if (V0 + V1 < 255) VDAC_EorCar_SetValue(engine);         //engine in bay
                    //else VDAC_EorCar_SetValue(255);  //if too big
                    
                    CyDelayUs(100*baytime[speedindex]);
                    
                    //VDAC_EorCar_SetValue(0);                  //engine not in bay  
                    
                    CyDelayUs(100*w2wtime[speedindex]);
                }    //end  for(j = 0; j<=3; j++) loop
            } 
            else
            {
                trainstart();                
            };//end if(passfreight)
        end comment out passenger or freight  */

/*
To activate a DMA channel from a CPU request, use this function:
            CyDmaChSetRequest(Channel_Handle, CPU_REQ);
            Channel_Handle is txChannel

To disable a DMA channel, use this function: 
            CyDmaChDisable(Channel_Handle);
            again, Channel_Handle is txChannel
            
*/
            // clear xp switch
            CyDmaChSetRequest(txChannel_0, CPU_REQ);

            //cars 
            for(j = 1; j<=carcount; j++)
            {
                //for each car, loop through each truck (2 trucks per car) for 4 wheels
                //fisrt wheel - bays 1 through 4 
                //first wheel - bays 5 through 8 and second wheel, bays 1 through 4
                //second wheel - bays 5 through 8
                //all bays reset to no train conditions
            
                //first wheel bays 1 to 4 
                
                CyDmaChSetRequest(txChannel_1, CPU_REQ);   //wheel in bay 1
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   

                CyDmaChSetRequest(txChannel_2, CPU_REQ);   //wheel in bay 2
                xp_parallel_clock();          
                CyDelayUs(100*baytime[speedindex]);        //time in bay   

                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   

                CyDmaChSetRequest(txChannel_3, CPU_REQ);   //wheel in bay 3
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   

                CyDmaChSetRequest(txChannel_4, CPU_REQ);   //wheel in bay 4
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                //first wheel - bays 5 through 8 and second wheel, bays 1 through 4
                CyDmaChSetRequest(txChannel_51, CPU_REQ);  //wheel in bay 5 & 1
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_62, CPU_REQ);  //wheel in bay 6 & 2
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_73, CPU_REQ);  //wheel in bay 7 & 3
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_84, CPU_REQ);  //wheel in bay 8 & 4
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                //second wheel - bays 5 through 8
                CyDmaChSetRequest(txChannel_5, CPU_REQ);   //wheel in bay 5
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_6, CPU_REQ);   //wheel in bay 6
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_7, CPU_REQ);   //wheel in bay 7
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_8, CPU_REQ);   //wheel in bay 8
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDelayUs(100*w2wtime[speedindex]);        //wheel to wheel time    

                //third wheel - bays 1 through 4
                CyDmaChSetRequest(txChannel_1, CPU_REQ);   //wheel in bay 1
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   

                CyDmaChSetRequest(txChannel_2, CPU_REQ);   //wheel in bay 2
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   

                CyDmaChSetRequest(txChannel_3, CPU_REQ);   //wheel in bay 3
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   

                CyDmaChSetRequest(txChannel_4, CPU_REQ);   //wheel in bay 4
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                //third wheel - bays 5 through 8 and fourth wheel, bays 1 through 4
                CyDmaChSetRequest(txChannel_51, CPU_REQ);  //wheel in bay 5 & 1
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_62, CPU_REQ);  //wheel in bay 6 & 2
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_73, CPU_REQ);  //wheel in bay 7 & 3
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_84, CPU_REQ);  //wheel in bay 8 & 4
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                //fourth wheel - bays 5 through 8
                CyDmaChSetRequest(txChannel_5, CPU_REQ);   //wheel in bay 5
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_6, CPU_REQ);   //wheel in bay 6
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_7, CPU_REQ);   //wheel in bay 7
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
            
                CyDmaChSetRequest(txChannel_8, CPU_REQ);   //wheel in bay 8
                xp_parallel_clock();
                CyDelayUs(100*baytime[speedindex]);        //time in bay   
                
                CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
                xp_parallel_clock();
                CyDelayUs(100*tietime[speedindex]);        //time in tie   
  
                CyDelayUs(100*t2ttime[speedindex]);        //time last last wheel of first truck to first wheel of second truck            
            
            }  //end of for(j = 1; j<=carcount; j++) - car count
               
        //end of train
        CyDmaChSetRequest(txChannel_0, CPU_REQ);   //no wheel in bay
        trainstop();
        
        //headway:
        for(ii = 1; ii <= headway*3 ; ii++)  //headway is in 15 second steps
        {
            if (endheadway) break;
            //CyDelay(5000);        // comment out 5 second delay
        } 
        endheadway = 0; 
        

    }   //end while (1)

} //end of main
/* END OF FILE */